@@ -20,19 +20,19 @@
 # Author: Aurelien Bompard <abompard@fedoraproject.org>
 #
 
+import hmac
 import json
 import logging
 from email import message_from_binary_file
 from email.message import EmailMessage
 from email.policy import default
 from functools import wraps
-from urllib.parse import urljoin
+from urllib.parse import unquote, urljoin
 
 from django.conf import settings
 from django.core.exceptions import ImproperlyConfigured, SuspiciousOperation
 from django.http import HttpResponse
 from django.urls import reverse
-from django.utils.http import urlunquote
 from django.views.decorators.csrf import csrf_exempt
 from django.views.decorators.http import require_POST
 
@@ -71,7 +71,43 @@ def key_and_ip_auth(func):
                  assigned to MAILMAN_ARCHIVER_FROM in the settings file.
                 </p></body></html>""",
                 content_type="text/html", status=403)
-        if request.GET.get("key") != settings.MAILMAN_ARCHIVER_KEY:
+
+        authorization = request.headers.get('Authorization')
+        if authorization is None:
+            if request.GET.get('key') is not None:
+                # Old authentication method used, need to upgrade
+                # mailman-hyperkitty
+                logger.error(
+                    'The MAILMAN_ARCHIVER_KEY is now required to be '
+                    'sent over the Authorization HTTP header, you need to '
+                    'upgrade the mailman-hyperkitty package to 1.2.0 or newer.'
+                )
+                return HttpResponse(
+                    """<html><title>Auth required</title><body>
+                    <h1>Authorization Required</h1><p>The archiver key is now
+                     required to be sent over the Authorization HTTP header.
+                     You need to upgrade the mailman-hyperkitty package to
+                     1.2.0 or newer.
+                    </p></body></html>""",
+                    content_type="text/html", status=401)
+            else:
+                # No auth provided over header nor GET param
+                logger.error(
+                    'The MAILMAN_ARCHIVER_KEY was not sent as the '
+                    'Authorization HTTP header.'
+                )
+                return HttpResponse(
+                    """<html><title>Auth required</title><body>
+                    <h1>Authorization Required</h1><p>The archiver key must be
+                     sent over the Authorization HTTP header.
+                    </p></body></html>""",
+                    content_type="text/html", status=401)
+
+        auth = authorization.split()
+        # Use timing-attack safe comparison of secret key
+        if (len(auth) != 2 or auth[0] != 'Token' or
+                not hmac.compare_digest(
+                    auth[1], settings.MAILMAN_ARCHIVER_KEY)):
             return HttpResponse(
                 """<html><title>Auth required</title><body>
                 <h1>Authorization Required</h1><p>Please check whether
@@ -97,7 +133,7 @@ def _get_url(mlist_fqdn, msg_id=None):
         msg_hash = get_message_id_hash(msg_id.strip().strip("<>"))
         url = reverse('hk_message_index', kwargs={
             "mlist_fqdn": mlist_fqdn, "message_id_hash": msg_hash})
-    relative_url = urlunquote(url)
+    relative_url = unquote(url)
     mail_domain = mlist_fqdn.split("@")[1]
     try:
         domain = MailDomain.objects.get(
