@@ -157,6 +157,9 @@ DATABASES = {
     #}
 }
 
+# Set default type of primary key (feature introduced with django 3.2)
+# https://docs.djangoproject.com/en/3.2/releases/3.2/#customizing-type-of-auto-created-primary-keys
+DEFAULT_AUTO_FIELD = 'django.db.models.AutoField'
 
 # If you're behind a proxy, use the X-Forwarded-Host header
 # See https://docs.djangoproject.com/en/1.8/ref/settings/#use-x-forwarded-host
@@ -369,6 +372,7 @@ REST_FRAMEWORK = {
 #
 Q_CLUSTER = {
     'timeout': 300,
+    'retry': 360,
     'save_limit': 100,
     'orm': 'default',
 }
